@@ -33,6 +33,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #ifndef HAVE_KRB_GET_ERR_TEXT
 #define krb_get_err_text(status) krb_err_txt[status]
 #endif
+int remember_krb_auth_ret = 0; /* yes, I like lisp. go away. */
 #endif
 
 /* For the GNATS server we'll be talking with.  */
@@ -230,6 +231,14 @@ get_reply ()
       safe_exit ();
       break;
 
+    case CODE_INVALID_STATE:
+      s = strchr (r->text, '.'); s[0] = '\0';
+      s = strrchr (r->text, ' ') + 1;
+      fprintf (stderr, "%s: no such state %s\n",
+	       program_name, s);
+      safe_exit ();
+      break;
+
     case CODE_INVALID_RESPONSIBLE:
       s = strchr (r->text, '.'); s[0] = '\0';
       s = strrchr (r->text, ' ') + 1;
@@ -280,7 +289,15 @@ get_reply ()
       fprintf (stderr, "%s: no Kerberos support, authentication failed\n",
 	       program_name);
     case CODE_NO_ACCESS:
+#ifdef HAVE_KERBEROS
+      if (remember_krb_auth_ret)
+	fprintf (stderr, "%s: error (%s) while attempting krb4-auth\n",
+		 program_name, krb_get_err_text (remember_krb_auth_ret));
+      else
+	fprintf (stderr, "%s: access denied\n", program_name);
+#else
       fprintf (stderr, "%s: access denied\n", program_name);
+#endif
       safe_exit ();
       break;
 
@@ -471,6 +488,7 @@ client_init ()
     ret = krb_mk_auth (KOPT_DO_MUTUAL, &k.tkt,
 		       GNATS_KRB4_PRINCIPAL_NAME, hname, realm,
 		       CKSUM, GNATS_KRB4_VERSIONID, &k.packet);
+    remember_krb_auth_ret = ret;
     if (ret)
       SKIP (krb_get_err_text (ret));
 
