@@ -1,5 +1,5 @@
 /* Commands for the GNATS server.
-   Copyright (C) 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1994, 95, 95, 1997 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
 
 This file is part of GNU GNATS.
@@ -26,6 +26,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 char *current_pr = (char *)NULL;
 
 #ifdef HAVE_KERBEROS
+#include <des.h>
 #include <krb.h>
 #ifndef HAVE_KRB_GET_ERR_TEXT
 #define krb_get_err_text(status) krb_err_txt[status]
@@ -42,19 +43,6 @@ extern int client_authorized;
 
 static int chk_nargs			PARAMS((int, char*));
 
-/* Only say `PRs follow' once.  */
-int started = 0;
-
-void
-start_data ()
-{
-  if (!started)
-    {
-      printf ("%d PRs follow.\r\n", CODE_PR_READY);
-      started = 1;
-    }
-}
-
 static int
 chk_nargs (ac, what)
      int ac;
@@ -131,6 +119,9 @@ qualify (ac, av)
 		sprintf (path, "%s/%s/%s", gnats_root, j->category, j->number);
 		av[i] = path;
 		found = 1;
+		/* Only go thru for a particular pattern until we make a
+		   match; once we do, move on to the next argument.  */
+		break;
 	      }
 	  }
     }
@@ -310,6 +301,8 @@ do_query (ac, av)
   Index *j;
   int hit = 0;
 
+  reset_started ();
+
   if (ac == 0)
     {
       /* We weren't given a list of PRs to check, so we do the
@@ -325,8 +318,6 @@ do_query (ac, av)
 	  }
       if (! hit)
 	printf ("%d No PRs match.\r\n", CODE_NO_PRS);
-      else
-	started = 0;
     }
   else if (qualify (ac, av))
     {
@@ -466,7 +457,6 @@ GNATS_lock (ac, av)
   if (! path)
     return;
 
-  start_data ();
   query_format = FORMAT_FULL;
   daemon_pr (path, 1);
   printf (".\r\n");
@@ -489,8 +479,8 @@ GNATS_unlk (ac, av)
 }
 
 int
-check_locked (path, pr)
-     char *path, *pr;
+check_locked (path, p)
+     char *path, *p;
 {
   struct stat buf;
   char *lock_path = (char *) xmalloc (strlen (path) + 6);
@@ -501,16 +491,53 @@ check_locked (path, pr)
     {
       if (errno == ENOENT)
 	printf ("%d You must lock PR %s and get its current content first.\r\n",
-		CODE_PR_NOT_LOCKED, pr);
+		CODE_PR_NOT_LOCKED, p);
       else
 	printf ("%d Could not detect if the lock for %s was there.\r\n",
-		CODE_FILE_ERROR, pr);
+		CODE_FILE_ERROR, p);
       return 0;
     }
   xfree (lock_path);
   return 1;
 }
 
+void
+GNATS_lkdb (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LKDB");
+      return;
+    }
+
+  /* This does the error for us.  */
+  if (lock_gnats () == 0)
+    printf ("%d GNATS database is now locked.\r\n", CODE_OK);
+}
+
+void
+GNATS_undb (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("UNDB");
+      return;
+    }
+
+  if (unlock_gnats () < 0)
+    {
+      printf ("%d failed unlocking GNATS database.\r\n",
+	      CODE_ERROR);
+      return;
+    }
+
+  printf ("%d GNATS database is now unlocked.\r\n", CODE_OK);
+}
+
 void
 GNATS_edit (ac, av)
      int ac;
@@ -518,6 +545,8 @@ GNATS_edit (ac, av)
 {
   char *given, *path;
   FILE *fp;
+  /* By default, unlock the PR after it's been filed.  */
+  int unlockp = 1;
 
   if (! chk_nargs (ac, "PR to be edited"))
     return;
@@ -558,11 +587,16 @@ GNATS_edit (ac, av)
     goto editout;
 
   if (! modify_pr (fp))
-    printf ("%d Failed writing out PR.\r\n", CODE_ERROR);
+    {
+      printf ("%d Failed writing out PR.\r\n", CODE_ERROR);
+      /* If we couldn't write it, we shouldn't unlock it.  */
+      unlockp = 0;
+    }
   else
     printf ("%d PR %s filed.\r\n", CODE_OK, av[0]);
   fclose (fp);
-  unlock_gnats ();
+  if (unlockp)
+    unlock_gnats ();
   unlock_pr (path); /* XXX */
   /* unblock_signals (); */
   if (unlink (given) && errno != ENOENT)
@@ -623,7 +657,8 @@ GNATS_mlpr (ac, av)
 	addrs[naddrs++] = submitter.contact;
       if (submitter.notify[0])
 	addrs[naddrs++] = submitter.notify;
-      addrs[naddrs++] = responsible->alias;
+      if (responsible)
+	addrs[naddrs++] = responsible->alias;
       h = raw_header_value (X_GNATS_NOTIFY);
       if (h)
 	{
@@ -632,6 +667,12 @@ GNATS_mlpr (ac, av)
 	}
       addrs[naddrs] = NULL;
 
+      if (responsible)
+	{
+	  free_responsible (responsible);
+	  xfree ((char *) responsible);
+	}
+
       i = unify_addrs (addrs, naddrs);
       if (i == naddrs)
 	{
@@ -649,8 +690,6 @@ GNATS_mlpr (ac, av)
 	    xfree (addrs[i]);
 	  }
       printf ("%d End of list.\r\n", CODE_INFORMATION);
-      if (responsible)
-	free_responsible (responsible);
     }
   else
     printf ("%d Invalid PR %s.\n", CODE_INVALID_PR, av[0]);
@@ -737,7 +776,7 @@ GNATS_resp (ac, av)
 {
   Responsible *responsible;
 
-  if (! chk_nargs (ac, "name"))
+  if (! chk_nargs (ac, "responsible entry"))
     return;
 
   responsible = get_responsible_address (av[0]);
@@ -748,6 +787,9 @@ GNATS_resp (ac, av)
       return;
     }
 
+  free_responsible (responsible);
+  xfree ((char *)responsible);
+
   searching = 1;
   search->responsible = (char *) strdup (av[0]);
   printf ("%d Ok.\r\n", CODE_OK);
@@ -778,6 +820,38 @@ GNATS_catg (ac, av)
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
+void
+GNATS_synp (ac, av)
+     int ac;
+     char **av;
+{
+  static char *buf = (char *)NULL;
+
+  if (! chk_nargs (ac, "synopsis"))
+    return;
+
+  if (buf)
+    xfree (buf);
+  synopsis = buf = strdup(*av);
+  printf ("%d Ok.\r\n", CODE_OK);
+}
+
+void
+GNATS_rlse (ac, av)
+     int ac;
+     char **av;
+{
+  static char *buf = (char *)NULL;
+
+  if (! chk_nargs (ac, "release"))
+    return;
+
+  if (buf)
+    xfree (buf);
+  release = buf = strdup(*av);
+  printf ("%d Ok.\r\n", CODE_OK);
+}
+
 void
 GNATS_conf (ac, av)
      int ac;
@@ -822,18 +896,15 @@ GNATS_orig (ac, av)
      int ac;
      char **av;
 {
-  int i, len = 0;
   static char *buf = (char *)NULL;
 
+  if (! chk_nargs (ac, "name"))
+    return;
+
   if (buf)
     xfree (buf);
-  for (i = 0; i < ac; i++)
-    len += strlen (av[i]);
-  buf = (char *) xmalloc (len + ac);
-  strcpy (buf, av[0]);
-  for (i = 1; i < ac; i++)
-    sprintf (buf, "%s %s", buf, av[i]);
-  originator = buf;
+  originator = buf = strdup(*av);
+
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -842,18 +913,15 @@ GNATS_clss (ac, av)
      int ac;
      char **av;
 {
-  int i, len = 0;
   static char *buf = (char *)NULL;
 
+  if (! chk_nargs(ac, "class list"))
+    return;
+
   if (buf)
     xfree (buf);
-  for (i = 0; i < ac; i++)
-    len += strlen (av[i]);
-  buf = (char *) xmalloc (len + ac);
-  strcpy (buf, av[0]);
-  for (i = 1; i < ac; i++)
-    sprintf (buf, "%s %s", buf, av[i]);
-  class = buf;
+  class = buf = strdup(*av);
+
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -863,18 +931,15 @@ GNATS_qrtr (ac, av)
      int ac;
      char **av;
 {
-  int i, len = 0;
   static char *buf = (char *)NULL;
 
+  if (! chk_nargs(ac, "quarter"))
+    return;
+
   if (buf)
     xfree (buf);
-  for (i = 0; i < ac; i++)
-    len += strlen (av[i]);
-  buf = (char *) xmalloc (len + ac);
-  strcpy (buf, av[0]);
-  for (i = 1; i < ac; i++)
-    sprintf (buf, "%s %s", buf, av[i]);
-  quarter = buf;
+  quarter = buf = strdup(*av);
+
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
@@ -883,20 +948,59 @@ GNATS_kywd (ac, av)
      int ac;
      char **av;
 {
-  int i, len = 0;
   static char *buf = (char *)NULL;
 
+  if (! chk_nargs(ac, "keyword list"))
+    return;
+
   if (buf)
     xfree (buf);
-  for (i = 0; i < ac; i++)
-    len += strlen (av[i]);
-  buf = (char *) xmalloc (len + ac);
-  strcpy (buf, av[0]);
-  for (i = 1; i < ac; i++)
-    sprintf (buf, "%s %s", buf, av[i]);
-  keywords = buf;
+  keywords = buf = strdup(*av);
+
   printf ("%d Ok.\r\n", CODE_OK);
 }
+
+void
+GNATS_bfor (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      required_before = t;
+      searching = 1;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_aftr (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      required_after = t;
+      searching = 1;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
 #endif
 
 void
@@ -912,73 +1016,122 @@ GNATS_prio (ac, av)
   printf ("%d Ok.\r\n", CODE_OK);
 }
 
-char *
-full_date (ac, av)
-     int ac;
-     char **av;
-{
-  int i, len = 0;
-  static char *buf = (char *)NULL;
-
-  if (buf)
-    xfree (buf);
-  for (i = 0; i < ac; i++)
-    len += strlen (av[i]);
-  buf = (char *) xmalloc (len + ac);
-  strcpy (buf, av[0]);
-  for (i = 1; i < ac; i++)
-    sprintf (buf, "%s %s", buf, av[i]);
-  return buf;
-}
-
 void
-GNATS_bfor (ac, av)
+GNATS_abfr (ac, av)
      int ac;
      char **av;
 {
-  char *s;
   time_t t;
 
-  if (ac == 0)
-    {
-      printf ("%d You must supply a date with BFOR.\n", CODE_ERROR);
-      return;
-    }
+  if (! chk_nargs(ac, "date"))
+    return;
 
-  s = full_date (ac, av);
-  t = get_date (s, NULL);
-  if (t == -1)
-    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, s);
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
-      before = t;
-      searching = 1;
+      arrived_before = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
 }
 
 void
-GNATS_aftr (ac, av)
+GNATS_araf (ac, av)
      int ac;
      char **av;
 {
-  char *s;
   time_t t;
 
-  if (ac == 0)
-    {
-      printf ("%d You must supply a date with BFOR.\n", CODE_ERROR);
-      return;
-    }
+  if (! chk_nargs(ac, "date"))
+    return;
 
-  s = full_date (ac, av);
-  t = get_date (s, NULL);
-  if (t == -1)
-    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, s);
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     {
-      after = t;
-      searching = 1;
+      arrived_after = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_cbfr (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      closed_before = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_caft (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      closed_after = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_mbfr (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      modified_before = t;
+      printf ("%d Ok.\r\n", CODE_OK);
+    }
+}
+
+void
+GNATS_maft (ac, av)
+     int ac;
+     char **av;
+{
+  time_t t;
+
+  if (! chk_nargs(ac, "date"))
+    return;
+
+  t = get_date (*av, NULL);
+  if (t == -1)
+    printf ("%d Cannot parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
+  else
+    {
+      modified_after = t;
       printf ("%d Ok.\r\n", CODE_OK);
     }
 }
@@ -988,16 +1141,14 @@ GNATS_vdat (ac, av)
      int ac;
      char **av;
 {
-  char *s;
   time_t t;
 
   if (! chk_nargs (ac, "date"))
     return;
 
-  s = full_date (ac, av);
-  t = get_date (s, NULL);
-  if (t == -1)
-    printf ("%d Couldn't parse the date: %s.\r\n", CODE_INVALID_DATE, s);
+  t = get_date (*av, NULL);
+  if (t < 0)
+    printf ("%d Couldn't parse the date: %s.\r\n", CODE_INVALID_DATE, *av);
   else
     printf ("%d Ok.\r\n", CODE_OK);
 }
@@ -1088,18 +1239,21 @@ GNATS_rset (ac, av)
   originator = (char *) NULL;
   xfree (class);
   class = (char *) NULL;
-  before = after = (time_t)0;
   searching = skip_closed = 0;
   xfree (text_search);
   text_search = (char *) NULL;
   xfree (m_text_search);
   m_text_search = (char *) NULL;
 #ifdef GNATS_RELEASE_BASED
+  required_before = required_after = (time_t)0;
   xfree (quarter);
   quarter = (char *) NULL;
   xfree (keywords);
   keywords = (char *) NULL;
 #endif
+  arrived_before  = arrived_after  = (time_t)0;
+  modified_before = modified_after = (time_t)0;
+  closed_before   = closed_after   = (time_t)0;
 
   if (new_index)
     printf ("%d Reset state...reloaded the index.\r\n", CODE_OK);
@@ -1178,12 +1332,8 @@ GNATS_auth (ac, av)
      int ac;
      char **av;
 {
-  if (ac != 1)
-    {
-      printf ("%d One authentication type name required.\r\n",
-	      CODE_ERROR);
-      return;
-    }
+  if (! chk_nargs(ac, "authentication type name"))
+    return;
 #ifdef HAVE_KERBEROS
   /* This isn't the right way to do this ... we ought to exchange data
      in printable ascii form or something like that.  This'll do for
@@ -1192,7 +1342,7 @@ GNATS_auth (ac, av)
     {
       extern char *getusershell ();
 
-      char user[ANAME_SZ], keyfile[PATH_MAX], desired_user[32];
+      char keyfile[PATH_MAX], desired_user[32];
       struct sockaddr_in peer, laddr;
       int len = sizeof (peer);
       int status;
@@ -1326,7 +1476,6 @@ GNATS_auth (ac, av)
       p = k.buf + len + 1;
       tohex (k.buf, p, len);
       printf ("x%s\r\n", p);
-      fflush (stdout);
 
       /* Okay, authentication step is done.  Now check authorization.  */
       if (kuserok (&k.auth, desired_user))
@@ -1411,6 +1560,51 @@ GNATS_lsub (ac, av)
     printf (".\r\n");
 }
 
+void
+GNATS_lsta (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LSTA");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_STATES, NULL) == 0)
+    printf (".\r\n");
+}
+
+void
+GNATS_lcla (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LCLA");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_CLASSES, NULL) == 0)
+    printf (".\r\n");
+}
+
+void
+GNATS_lcfg (ac, av)
+     int ac;
+     char **av;
+{
+  if (ac)
+    {
+      noargs ("LCFG");
+      return;
+    }
+  printf ("%d List follows.\r\n", CODE_PR_READY);
+  if (get_gnats_file (LIST_CONFIG, NULL) == 0)
+    printf (".\r\n");
+}
+
 void
 GNATS_help (ac, av)
      int ac;
@@ -1428,14 +1622,24 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   LSUB                    list the available submitters\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   LSTA                    list the available states\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   LCLA                    list the available classes\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   LCFG                    list the GNATS config file\r\n",
+	  CODE_INFORMATION);
   printf ("%d-   CATG <category>         search for PRs in <category>\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   SYNP <synopsis>         search for PRs have the synopsis <synopsis>\r\n",
+	  CODE_INFORMATION);
   printf ("%d-   RESP <person>           search for PRs that <person> is responsible for\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   SUBM <submitter>        search for PRs with a submitter <submitter>\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   ORIG <originator>       search for PRs with an originator <originator>\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   RLSE <release>          search for PRs from the release <release>\r\n",
+	  CODE_INFORMATION);
   printf ("%d-   CLSS <class>            search for PRs of class <class>\r\n",
 	  CODE_INFORMATION);
 #ifdef GNATS_RELEASE_BASED
@@ -1443,7 +1647,27 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   KYWD <keywords>         search for PRs with Keywords <keywords>\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   BFOR <date>             find PRs with Date-Required: before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   AFTR <date>             find PRs with Date-Required: after <date>\r\n",
+	  CODE_INFORMATION);
 #endif
+  printf ("%d-   PRIO <priority>         search for PRs which are of the priority <priority>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   ABFR <date>             find all PRs that arrived before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   ARAF <date>             find all PRs that arrived before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   MBFR <date>             find all PRs that were modified before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   MAFT <date>             find all PRs that were modified before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CBFR <date>             find all PRs that were closed before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   CAFT <date>             find all PRs that were closed before <date>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   VDAT <date>             validate the format of <date>\r\n",
+	  CODE_INFORMATION);
   printf ("%d-   SVTY <sevty>            search for PRs which are of the severity <sevty>\r\n",
 	  CODE_INFORMATION);
   printf ("%d-   STAT <state>            search for PRs in the state <state>\r\n",
@@ -1464,6 +1688,16 @@ GNATS_help (ac, av)
 	  CODE_INFORMATION);
   printf ("%d-   RSET                    reset internal settings to initial startup\r\n",
 	  CODE_INFORMATION);
+  printf ("%d-   LKDB                    lock the main GNATS database\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   UNDB                    unlock the main GNATS database\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   LOCK <PR> <user> <pid>  lock <PR> for <user> and optional <pid> and return PR text\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   UNLK <PR>               unlock <PR>\r\n",
+	  CODE_INFORMATION);
+  printf ("%d-   EDIT <PR>               check in edited <PR>\r\n",
+	  CODE_INFORMATION);
 #ifdef HAVE_KERBEROS
   printf ("%d-   AUTH <kerberos>	 Kerberos authentication\r\n",
 	  CODE_INFORMATION);
