@@ -235,6 +235,28 @@ modify_pr (fp)
   xfree (pr[LAST_MODIFIED].value);
   pr[LAST_MODIFIED].value = date;
 
+  if (! force)
+    {
+      if (strcmp (old_index->state, new_index->state) != 0 &&
+          strcmp (get_state_type (old_index->state),
+                  get_state_type (new_index->state)) != 0)
+        {
+          /* The state and state type have changed */
+          if (check_state_type (old_index->state, "closed"))
+            {
+              /* No longer closed; clear the Closed-Date field */
+              xfree (pr[CLOSED_DATE].value);
+              pr[CLOSED_DATE].value = strdup("");
+            }
+          else if (check_state_type (new_index->state, "closed"))
+            {
+              /* Now closed; set the Closed-Date field */
+               xfree (pr[CLOSED_DATE].value);
+               pr[CLOSED_DATE].value = date;
+            }
+        }
+    }
+  
   write_header (prfile, NUM_HEADER_ITEMS);
   fprintf (prfile, "\n");
   write_pr (prfile, NUM_PR_ITEMS);
@@ -353,10 +375,11 @@ check_pr (fp)
   if (d)
     {
       t = get_date (d, NULL);
-      if (! t)
+      if (t < 0)
 	{
 	  fprintf (stderr,
 		   "%s: Couldn't parse the date: %s.\r\n", program_name, d);
+	  return 0;
 	}
     }
 #endif
