@@ -25,18 +25,11 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 static int verify_analysis	PARAMS((void));
 static void run_atpr		PARAMS((Submitter *, struct tm *, char *, char *));
-static void free_category	PARAMS((Category *));
-static void free_submitter	PARAMS((Submitter *));
 static void reply_to_submitter	PARAMS((char *));
 static void create_report	PARAMS((char *, int));
 static int get_bug_number	PARAMS((void));
 static void notify_responsible	PARAMS((char*, char*, char*, char*, char*, int, struct tm *));
-
-void
-start_data ()
-{
-  return; /* Nothing */
-}
+static char* derive_submitter  PARAMS((void));
 
 static struct bad_enum *bad_enums = NULL;
 
@@ -51,15 +44,17 @@ gnats (fp)
   char number[14]; /* No more than 14 digits in a PR #, ha ha.  */
   char bug_name[STR_MAX];
   Responsible *responsible = NULL;
-  char arrival_time[30];
+  char arrival_time[GNATS_TIME_LENGTH];
   struct tm *expired = NULL;
 
   char *site, *bug_group;
   char *subject = NULL, *synopsis = NULL;
   char *p;
+  char message[STR_MAX];
+  char *from_address;
 
   mode_t mode;
-  struct stat buf;
+  struct stat sbuf;
   time_t seconds;
 
   /* Zero things out to make sure.  */
@@ -69,17 +64,27 @@ gnats (fp)
   if (fp == (FILE *) NULL)
     return;
 
-  if (stat (gnats_root, &buf) == -1)
+  if (stat (gnats_root, &sbuf) == -1)
     punt (1, "Can't read gnats_root (looking at %s).", gnats_root);
 
   /* Read the PR in. */
   read_pr (fp, 0);
   fclose (fp);
 
+/*  If we don't have a valid submitter ID, try to get it from the "From:"
+    header.  If that doesn't work, use the default from the config file.  */
+
   site = field_value (SUBMITTER);
+  if ((find_submitter (&submitter, site) == -1) 
+      || (strcmp (site, def_subm) == 0))
+  {
+    site = derive_submitter (); 
+    set_field (SUBMITTER, site);
+  }
 
   if (find_submitter (&submitter, site) == -1)
     {
+
       log_msg (LOG_INFO, 1, "resetting to default submitter from:", site);
       site = def_subm;
       set_field (SUBMITTER, def_subm);
@@ -92,15 +97,16 @@ gnats (fp)
   bug_group = field_value (CATEGORY);
   if (find_category (&category, bug_group) == -1)
     {
-      log_msg (LOG_INFO, 1, "resetting bug category to pending from:",
-	       bug_group);
-      bug_group = PENDING;
-      set_field (CATEGORY, PENDING);
+      sprintf (message, "%s from: %s\n", default_category, bug_group);
+      log_msg (LOG_INFO, 1, "resetting bug category to ", message);
+      bug_group = default_category;
+      set_field (CATEGORY, default_category);
 
-      /* Again, but with `pending' now.  If it's not there, kick this to
+      /* Again, but with default now.  If it's not there, kick this to
 	 the GNATS_ADMIN.  */
       if (find_category (&category, bug_group) == -1)
-	punt (1, "No `pending' directory under gnats-root (%s).", gnats_root);
+	punt (1, "No `%s' directory under gnats-root (%s).", 
+	      default_category, gnats_root);
     }
 
   /* Set responsible field, adding full name as well (if found).  If it's
@@ -182,7 +188,8 @@ gnats (fp)
 
   /* Set arrival date and time of response.  */
   seconds = time ((time_t) 0);
-  strftime (arrival_time, 30, "%a %b %e %T %Z %Y", localtime (&seconds));
+  strftime (arrival_time, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y",
+	    localtime (&seconds));
   set_field (ARRIVAL_DATE, arrival_time);
 
   /* If the submitter isn't supposed to have a response time, don't bother
@@ -191,34 +198,35 @@ gnats (fp)
     expired = get_response_time (submitter.rtime);
 
   /* Put together the path to where the bug will be stored.  If the dir
-     is not there, and the category is PENDING, auto-create that one,
+     is not there, and the category is the default, auto-create that one,
      if we want to.  If not, make the bug pending, and store in there.  */
   sprintf (path, "%s/%s", gnats_root, bug_group);
-  err = stat (path, &buf);
+  err = stat (path, &sbuf);
   if (err == -1 && !flag_autocreate)
     {
-      bug_group = PENDING;
-      set_field (CATEGORY, PENDING);
-      log_msg (LOG_INFO, 1, "directory does not exist, changing to pending:",
+      bug_group = default_category;
+      set_field (CATEGORY, default_category);
+      log_msg (LOG_INFO, 1, "directory does not exist, changing to default:",
 	       path);
       sprintf (path, "%s/%s", gnats_root, bug_group);
-      err = stat (path, &buf);
+      err = stat (path, &sbuf);
     }
 
-  /* Check ERR again, to see if pending was there.  */
+  /* Check ERR again, to see if default category was there.  */
   if (err == -1)
     {
-      if (strcmp (bug_group, PENDING) == 0)
+      if (strcmp (bug_group, default_category) == 0)
 	{
-	  log_msg (LOG_INFO, 0, "pending does not exist, creating...");
+	  log_msg (LOG_INFO, 1, default_category, 
+		   " does not exist, creating...");
 #ifdef S_IRWXU
 	  mode = S_IRWXU | S_IRGRP | S_IXGRP;
 #else
 	  mode = 0750;
 #endif
 	  if (mkdir (path, mode) == -1)
-	    punt (1, "Can't create `pending' directory under gnats_root (%s).",
-		  gnats_root);
+	    punt (1, "Can't create `%s' directory under gnats_root (%s).", 
+		  default_category, gnats_root);
 	}
       else if (flag_autocreate)
 	{
@@ -251,7 +259,7 @@ gnats (fp)
   log_msg (LOG_INFO, 1, "PR written out:", path);
 
   /* Add a line into the index for use by query-pr.  */
-  /* This should be done regardless of whether the bug is pending or not.  */
+  /* This should be done regardless of whether the bug is default or not.  */
   add_to_index ();
 
   /* If it fell into pending, don't send all of the mail for it, don't create
@@ -277,7 +285,8 @@ gnats (fp)
 
   free_category (&category);
   free_submitter (&submitter);
-/*  free_responsible (responsible); */ /* FIXME */
+  free_responsible (responsible);
+  xfree ((char *)responsible);
 }
 
 static int
@@ -322,8 +331,8 @@ run_atpr (submitter, expired, bug_name, path)
      char *bug_name, *path;
 {
   char *at_pr, *command;
-  char buf[STR_MAX];
-  FILE *pipe;
+  char buf[GNATS_TIME_LENGTH];
+  FILE *p;
   int len, i;
   static char *ats[] = { "/usr/bin/at", "/bin/at", NULL };
 
@@ -331,7 +340,7 @@ run_atpr (submitter, expired, bug_name, path)
   strcpy (at_pr, bindir);
   strcat (at_pr, "/at-pr");
 
-  len = strftime (buf, STR_MAX, "%H:%M %b %d", expired);
+  len = strftime (buf, GNATS_TIME_LENGTH, "%H:%M %b %d", expired);
 
   for (i = 0; ats[i]; i++)
     {
@@ -349,8 +358,8 @@ run_atpr (submitter, expired, bug_name, path)
       return;
     }
 
-  pipe = popen (command, "w");
-  if (pipe == (FILE *)NULL)
+  p = popen (command, "w");
+  if (p == (FILE *)NULL)
     {
       punt (0, "Couldn't open the pipe for at-pr.");
       log_msg (LOG_INFO, 0, "popen failed:");
@@ -361,11 +370,11 @@ run_atpr (submitter, expired, bug_name, path)
       char *cont = arg_quote (submitter->contact);
       char *pa = arg_quote (gnats_admin);
       
-      fprintf (pipe, "%s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
+      fprintf (p, "%s %d %s %s %s '%s' '%s' '%s'\n", at_pr,
 	       submitter->rtime, bug_name, path, submitter->key,
 	       orig, cont, pa);
 
-      pclose (pipe);
+      pclose (p);
       xfree (orig);
       xfree (cont);
       xfree (pa);
@@ -380,10 +389,11 @@ check_if_reply ()
   char token[STR_MAX];
   char *path;
   struct stat buf;
-  char *s, *c;
+  Category cat;
+  char *s, *c, *cat_name;
   struct re_pattern_buffer regex;
   struct re_registers regs;
-  int i, start, end;
+  int i, start, end, res;
   char *case_fold;
 
   s = header_value (SUBJECT);
@@ -403,7 +413,7 @@ check_if_reply ()
   regex.translate = case_fold;
 
   /* too restrictive? */
-#define PAT "(.*re[ \t]*:)?[ \t]*([-a-z0-9_+.]*/([0-9]+))"
+#define PAT "(.*re[ \t]*(\\[[0-9]+\\])?:)?[ \t]*([-a-z0-9_+.]*/([0-9]+))"
   re_compile_pattern (PAT, strlen (PAT), &regex);
   i = re_match (&regex, s, strlen (s), 0, &regs);
 
@@ -412,8 +422,8 @@ check_if_reply ()
   if (i <= 0)
     return NULL;
 
-  start = regs.start[2];
-  end = regs.end[2];
+  start = regs.start[3];
+  end = regs.end[3];
 
   xfree ((char *)regs.start);
   xfree ((char *)regs.end);
@@ -422,7 +432,35 @@ check_if_reply ()
 
   c = strchr (s + start, '/');
   if (c)
-    path = get_category (c + 1);
+    {
+      /* The category's name is what comes before the '/'. */
+      cat_name  = s + start;
+
+      /* Terminate it temporarily: */
+      *c = '\0';
+
+      /* Void cat just in case free_category() expects NULLs. */
+      memset ((void *) &cat, 0, sizeof (Category));
+
+      /* See if the category exists: */
+      res = find_category (&cat, cat_name);
+
+      /* Undo temporary terminator: */
+      *c = '/';
+
+      /* If no such category, then this is not a reply to a valid
+       * problem report.  This situtation can arise, for example, when
+       * someone has the string "OS/2" in their Subject header.
+       */
+      if (res == -1)
+        return NULL;
+      else
+        {
+          /* We only needed res, never cat, so free cat. */
+          free_category (&cat);
+          path = get_category (c + 1);
+        }
+    }
   else
     {
       path = xmalloc (PATH_MAX);
@@ -436,7 +474,87 @@ check_if_reply ()
 
   return path;
 }
-
+
+/*  This function compares the e-mail address in the "From:" header of a new
+    PR with entries in the "addresses" file.  If there is a match, we return
+    the submitter ID from "addresses", otherwise we return the default
+    submitter.   */
+
+static char *
+derive_submitter ()
+{
+  char *from_address;
+  char from_string[STR_MAX];
+  char *token = (char *) xmalloc (STR_MAX);
+  char path[STR_MAX];
+  char *compare;
+
+  char *name, *alias, *s, *t;
+  int i, start, end;
+
+  FILE *fp;
+
+  from_address = header_value (FROM);
+  strcpy (from_string, from_address);
+
+  if ((*from_string == '\0') || (*from_string == '\n'))
+  {
+    xfree (token);
+    return def_subm;
+  }
+  
+  s = from_string + strlen(from_string) - 2;
+  if (*s == '>')
+  {
+    *s = '\0';
+    for (t = s; *t != '<'; t--)
+      *t = toupper (*t);
+  }
+  else
+    for (t = from_string; *t != '\0'; t++)
+    {
+      *t = toupper (*t);
+      if ((*t == ' ') || (*t == '\n'))
+	*t = '\0';
+    }
+  sprintf (path, "%s/gnats-adm/addresses", gnats_root);
+  if ((fp = fopen (path, "r")) == (FILE *) NULL)
+  {
+    xfree (token);
+    return def_subm;
+  }
+  while ((fgets (token, STR_MAX, fp)) != NULL)
+    if (*token != '#')
+    {
+      name = token;
+      
+      for (alias = token; (*alias != ':') && (*alias != '\0'); alias++);
+      if (*alias == ':')
+      {
+	*alias = '\0';
+	if (*(++alias) != '\0')
+	  for (t = alias; *t != '\0'; t++)
+	  {
+	    *t = toupper (*t);
+	    if (*t == '\n')
+	      *t = '\0';
+	  }
+      }
+
+      compare = from_string + strlen (from_string) - strlen (alias);
+      if (strcmp (compare, alias) == 0)
+      {
+	fclose (fp);
+	return name;
+      }
+    }
+
+  xfree (token);
+  fclose (fp);
+  return def_subm;
+}
+
+
 /* Send a reply to the submitter of the bug.  */
 
 static void
@@ -563,7 +681,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
   FILE *msg;
   char *notify = (char *) alloca (BUFSIZ); /* FIXME - huge */
   char *subc = NULL, *subn = NULL, *cnote = NULL;
-  char buf[STR_MAX];
+  char buf[GNATS_TIME_LENGTH];
   Header_Name i;
 
   notify[0] = '\0';
@@ -618,7 +736,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
     fprintf(msg, "Resent-Cc: %s\n", notify);
   {
     char * reply_to = header_value (REPLY_TO);
-    if (! reply_to)
+    if (! (reply_to && *reply_to))
       reply_to = header_value (FROM);
     fprintf(msg, "Resent-Reply-To: %s, %s", gnats_addr, reply_to);
   }
@@ -642,7 +760,7 @@ notify_responsible (responsible, subcontact, subnotify, cnotify,
 
   if (flag_notify && rtime != -1 && expired && verify_analysis ())
     {
-      int len = strftime (buf, STR_MAX, "%a %b %d %X %Z %Y", expired);
+      int len = strftime (buf, GNATS_TIME_LENGTH, "%a %b %d %X %Z %Y", expired);
       fprintf (msg,
 	       "\n\tThe contract type is `%s' with a response time of %d business hours.\n",
 	       type, rtime);
@@ -696,6 +814,22 @@ The filename for it is %s.", filename);
   unblock_signals ();
 }
 
+
+/* Get a pr number from its number (just uses basename).
+   Result is allocated, caller may free. */
+static char *
+pr_number_from_path (path)
+     char *path;
+{
+  char *result;
+  result = (char *) basename (path);
+
+  if (result != NULL)
+    return strdup (result);
+  else
+    return NULL;
+}
+
 /* Append a reply to an existing report.  The entire body of the message
    is appended, while just the To:, From:, Date:, and Subject: lines from
    the header are included.  We used to send notification of the reply
@@ -715,7 +849,8 @@ append_report (infile, filename)
   Responsible *responsible;
   Submitter submitter;
   char *p, *q;
-  char *from, *to, *cc = " ", *subject = " ", *date;
+  char *number;
+  char *from, *to, *cc = " \n", *subject = " \n", *date;
   time_t t;
 
   /* Save the values we read while in main().  */
@@ -731,8 +866,8 @@ append_report (infile, filename)
   else
     {
       t = time((time_t) 0);
-      date = (char *) xmalloc (30);
-      strftime (date, 30, "%a, %d %b %y %T %Z\n", localtime(&t));
+      date = (char *) xmalloc (GNATS_TIME_LENGTH);
+      strftime (date, GNATS_TIME_LENGTH, "%a, %d %b %y %T %Z\n", localtime(&t));
     }
 
   /* Now read in the PR itself.  */
@@ -752,7 +887,7 @@ append_report (infile, filename)
   *b = '\0';
 
   /* Separate this from the rest of the audit trail.  */
-  APPEND_STRING ("\n", buf, b, buf_len, buf_max, l, 0);
+  APPEND_STRING ((char*)"\n", buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "From: %s", from);
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "To: %s", to);
@@ -763,12 +898,12 @@ append_report (infile, filename)
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
   sprintf (line, "Date: %s", date);
   APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
-  APPEND_STRING ("\n", buf, b, buf_len, buf_max, l, 0);
+  APPEND_STRING ((char*)"\n", buf, b, buf_len, buf_max, l, 0);
 
   while ((fgets (line, STR_MAX, infile)) != NULL)
     {
       /* Quote all correspondence so PR doesn't get messed up */
-      APPEND_STRING (" ", buf, b, buf_len, buf_max, l, 0);
+      APPEND_STRING ((char*)" ", buf, b, buf_len, buf_max, l, 0);
       APPEND_STRING (line, buf, b, buf_len, buf_max, l, 0);
     }
   buf[buf_len] = '\0';
@@ -788,8 +923,8 @@ append_report (infile, filename)
   /* Set the >Last-Modified: field.  */
   t = time ((time_t) 0);
   xfree (date);
-  date = (char *) xmalloc (30);
-  strftime (date, 30, "%a %b %e %T %Z %Y", localtime (&t));
+  date = (char *) xmalloc (GNATS_TIME_LENGTH);
+  strftime (date, GNATS_TIME_LENGTH, "%a %b %e %T %Z %Y", localtime (&t));
   xfree (pr[LAST_MODIFIED].value);
   pr[LAST_MODIFIED].value = date;
 
@@ -812,22 +947,24 @@ append_report (infile, filename)
     punt (1, "%s: cannot open mail agent.", program_name);
   fprintf (msg, "To: %s\n", responsible->alias);
 
+  number = pr_number_from_path (filename);
+
   if (find_submitter (&submitter, field_value (SUBMITTER)) == -1)
     punt (0, "%s: PR %s has a Submitter field of `%s', which is invalid.",
-	  program_name, field_value (NUMBER), field_value (SUBMITTER));
+	  program_name, number, field_value (SUBMITTER));
   else
     {
       char *addr;
-      char *cc_buf, *cc;
+      char *cc_buf, *new_cc;
       int cc_len, cc_max;
       
-      cc = cc_buf = (char *) xmalloc (BUFSIZ);
+      new_cc = cc_buf = (char *) xmalloc (BUFSIZ);
       cc_len = 0;
       cc_max = BUFSIZ;
       
-      *cc = '\0';
+      *new_cc = '\0';
       addr = append_notify (submitter.contact);
-      APPEND_STRING (addr, cc_buf, cc, cc_len, cc_max, l, 1);
+      APPEND_STRING (addr, cc_buf, new_cc, cc_len, cc_max, l, 1);
       cc_buf[cc_len] = '\0';
 
       fprintf (msg, "Cc: %s\n", cc_buf);
@@ -840,7 +977,7 @@ append_report (infile, filename)
   fprintf (msg, "\n");
 
   fprintf (msg, "The following reply was made to PR %s/%s; it has been noted by GNATS.\n",
-	   field_value (CATEGORY), field_value (NUMBER));
+	   field_value (CATEGORY), number);
 
   if (fwrite (buf, sizeof (char), buf_len, msg) != buf_len)
     {
@@ -899,36 +1036,4 @@ get_bug_number ()
 
   return bug_number;
 }
-
-/* Avoid memory leaks.  */
-static void
-free_submitter (submitter)
-     Submitter *submitter;
-{
-  if (submitter->key)
-    xfree (submitter->key);
-  if (submitter->fullname)
-    xfree (submitter->fullname);
-  if (submitter->type)
-    xfree (submitter->type);
-  if (submitter->contact && submitter->contact != gnats_admin)
-    xfree (submitter->contact);
-  if (submitter->notify && submitter->notify != gnats_admin)
-    xfree (submitter->notify);
-  return;
-}
 
-static void
-free_category (category)
-     Category *category;
-{
-  if (category->key)
-    xfree (category->key);
-  if (category->fullname)
-    xfree (category->fullname);
-  if (category->person)
-    xfree (category->person);
-  if (category->notify && category->notify != gnats_admin)
-    xfree (category->notify);
-  return;
-}
