@@ -99,9 +99,9 @@ static Command cmds[] =
   { "RESP", GNATS_resp, FLAG_NEED_AUTHORIZATION },
   { "CATG", GNATS_catg, FLAG_NEED_AUTHORIZATION },
   { "SYNP", GNATS_synp, FLAG_NEED_AUTHORIZATION },
+  { "STAT", GNATS_stat, FLAG_NEED_AUTHORIZATION },
   { "CONF", GNATS_conf, 0 },
   { "SVTY", GNATS_svty, 0 },
-  { "STAT", GNATS_stat, 0 },
   { "ORIG", GNATS_orig, FLAG_NEED_AUTHORIZATION|FLAG_ONE_ARG },
   { "RLSE", GNATS_rlse, FLAG_NEED_AUTHORIZATION },
   { "CLSS", GNATS_clss, FLAG_ONE_ARG },
@@ -116,11 +116,18 @@ static Command cmds[] =
 #endif
   { "ABFR", GNATS_abfr, FLAG_ONE_ARG },
   { "ARAF", GNATS_araf, FLAG_ONE_ARG },
+  { "CBFR", GNATS_cbfr, FLAG_ONE_ARG },
+  { "CAFT", GNATS_caft, FLAG_ONE_ARG },
+  { "MBFR", GNATS_mbfr, FLAG_ONE_ARG },
+  { "MAFT", GNATS_maft, FLAG_ONE_ARG },
   { "VDAT", GNATS_vdat, FLAG_ONE_ARG },
 
   { "LCAT", GNATS_lcat, FLAG_NEED_AUTHORIZATION },
+  { "LCFG", GNATS_lcfg, FLAG_NEED_AUTHORIZATION },
+  { "LCLA", GNATS_lcla, FLAG_NEED_AUTHORIZATION },
   { "LRES", GNATS_lres, FLAG_NEED_AUTHORIZATION },
   { "LSUB", GNATS_lsub, FLAG_NEED_AUTHORIZATION },
+  { "LSTA", GNATS_lsta, FLAG_NEED_AUTHORIZATION },
 
   /* Stop and start the database.  */
   { "LKDB", GNATS_lkdb, FLAG_NEED_AUTHORIZATION },
